"""
This script will test the scaling and performance of the code on your system.

It requires some user input, which you can define at the top of the script after the imports.

The following values should be configured according to your system:
-

It will then run the population you specified, first linearly <AMT_REPEATS> times,
and then using multiprocessing it will run the population <AMT_REPEATS> times each time
with more cores. (Up until <AMOUNT_OF_CPUS>)

TODO: get the real evolution time instead of the total as well
TODO: put the methods in functions and put them in a different file
"""

import os
import json
import socket
import psutil

import numpy as np

from binarycpython.utils.grid import Population
from scaling_functions import get_mp_results, get_linear_results

AMT_REPEATS = 5                         # Number of times the population will be repeated per cpu
                                        # number. Useful to get some reliable statistics
RESOLUTION = {"M_1": 50, "per": 60}     # Resolution of sampling of the population
RESULT_DIR = "scaling_results"          # Directory where the results are written to.
PLOT_DIR = "scaling_plots"              # Directory where the plots will be stored
TESTCASE = "linear vs MP batched"       # `name` of the calculation
STEPSIZE_CPUS = 1                       # Stepsize for the cpu number generator. Try to keep this
                                        # low, to get the most reliable results

AMOUNT_OF_CPUS = 4                                 # Amount of logical cpus the machine has.
# AMOUNT_OF_CPUS = psutil.cpu_count()

AMOUNT_OF_CORES = 2                                 # The amount of physical cores. This value
                                                    # is not vital bit will be used in the plot
# AMOUNT_OF_CORES = psutil.cpu_count(logical=False)   # You can also use the psutil function to get 
                                                    # the amt of physical cores, but this isnt fully
                                                    # reliable (in mar 2020 it didnt get this value
                                                    # right when there were multiple sockets)


# Some calculated values
TOTAL_SYSTEMS = int(np.prod([el for el in RESOLUTION.values()]))
HOSTNAME = socket.gethostname()


# Generate the range of cpu numbers
CPU_LIST = np.arange(1, AMOUNT_OF_CPUS+1, STEPSIZE_CPUS)
if not CPU_LIST[-1] == AMOUNT_OF_CPUS:
    CPU_LIST = np.append(CPU_LIST, np.array([AMOUNT_OF_CPUS]))

##################################################################
# Create dictionairy in which to store all the results:
result_dict = {}

#
result_dict["amt_systems"] = TOTAL_SYSTEMS
result_dict["hostname"] = HOSTNAME
result_dict["amt_logical_cores"] = AMOUNT_OF_CPUS
result_dict["amt_of_physical_cores"] = AMOUNT_OF_CORES
result_dict["testcase"] = TESTCASE

#################
# Configuring population
test_pop = Population()

test_pop.set(
    verbose=1, binary=1,
)

test_pop.add_grid_variable(
    name="M_1",
    longname="log primary mass",
    valuerange=[1, 100],
    resolution="{}".format(RESOLUTION["M_1"]),
    spacingfunc="const(1, 100, {})".format(RESOLUTION["M_1"]),
    probdist="Kroupa2001(M_1)",
    # probdist='self.custom_options["extra_prob_function"](M_1)',
    dphasevol="dlnm1",
    parameter_name="M_1",
    condition="",
)

test_pop.add_grid_variable(
    name="period",
    longname="period",
    valuerange=["M_1", 20],
    resolution="{}".format(RESOLUTION["per"]),
    spacingfunc="np.linspace(1, 10, {})".format(RESOLUTION["per"]),
    precode="orbital_period = period**2",
    probdist="flat(orbital_period)",
    parameter_name="orbital_period",
    dphasevol="dper",
    condition='self.grid_options["binary"]==1',
)

#######################################################################################
# Execute grids

# Linear runs
LINEAR_TIMES = get_linear_results(test_pop, AMT_REPEATS, TOTAL_SYSTEMS)
result_dict["linear"] = LINEAR_TIMES

#######################################################################################
# MP runs
MP_DICT = get_mp_results(test_pop, CPU_LIST, AMT_REPEATS, TOTAL_SYSTEMS)
result_dict["mp"] = MP_DICT

# Write to file and make sure the directory exists.
os.makedirs(RESULT_DIR, exist_ok=True)
with open(os.path.join(RESULT_DIR, "{}_{}_systems.json".format(HOSTNAME, TOTAL_SYSTEMS)), "w") as f:
    f.write(json.dumps(result_dict))
